require 'keybinding'
require 'physics'
require 'character_behaviour'

function love.load()

	math.randomseed( os.time() )
	--key values
	TileSize = 32
	TilesetX, TilesetY = 512, 1024
	SizeX, SizeY = 100, 100
	ScreenSizeX, ScreenSizeY = 32, 20
	
	--scrolling
	OffsetX, OffsetY = 9,13					--IN TILE UNITS!!
	SmoothOffsetX, SmoothOffsetY = 0,0 		-- in pixels
	OffsetBorder = 7	
	Scrolling_Value = 1						--how many tiles does it jump?
	
	--general variables
	x,y,z,i,j,k = 0,0,0,0,0,0
	MovSet = { "up", "down", "left", "right" }

	--game variables
	game_clock = 0
	
	--HUD elements
	Picset = {}
	Picset = {
		love.graphics.newImage('HUD_A.png'),
	}
	
	--creating tileset
	TilesetPic = love.graphics.newImage('TILESET.png')
	Tileset = { }
	
	for j=1, TilesetX/TileSize do
		for i=1, TilesetY/TileSize do 
			Tileset[((i-1)*(TilesetX/TileSize))+j] = love.graphics.newQuad((j-1)*TileSize, (i-1)*TileSize, TileSize, TileSize, TilesetX, TilesetY)
		end
	end	
	
	--uploading field
	Field = { }
	
	for j=1,SizeX do
	
		Field[j] = { }
	
		for i=1, SizeY do
			Field[j][i] = math.random(1,1)
		end
		
	end
	
	-- player
	Player = {
	X_Tile = 20, Y_Tile = 22, 
	Sprite = 9,
	Is_Moving = false, Mov_Dir = 'no',
	OffsetX = 0, OffsetY = 0,
	Inventory = {}, Handslots = {}
	}
	
	-- character
	Characters = {}
	
	for i=1, 15 do
		Characters[i] = {
			X_Tile = 17+i, Y_Tile = 14+i, 
			OffsetX = 0, OffsetY = 0,				
			Sprite = 12,	
			Fraction = "Streetknight", Type = "Test Puppet",
			Is_Moving = false, 
			Mov_Dir = "no", Mov_Spd = 2,
			State = "Patrolling",
			
			--variables for certain states
			--this is for Patrolling
			Patrol_Direction = "right",
			Patrol_Distance = 5,
			Patrol_Storing = 0
		}
	end
	
end

function love.draw()

	for j=1,ScreenSizeX do
		for i=1, ScreenSizeY do
			love.graphics.draw(TilesetPic, Tileset[Field[OffsetX+j][OffsetY+i]], SmoothOffsetX+(j-1)*TileSize, SmoothOffsetY+(i-1)*TileSize)
			--love.graphics.print(OffsetX+j.."\n"..OffsetY+i,(j-2)*TileSize, (i-1)*TileSize)
		end
	end
	
	--draw player
	love.graphics.draw(TilesetPic, Tileset[Player.Sprite], ((Player.X_Tile-1-OffsetX)*TileSize)+Player.OffsetX, ((Player.Y_Tile-1-OffsetY)*TileSize)+Player.OffsetY)
	
	--draw charcters, IF THEY ARE ON SHOWED FIELD -> they won't appear inside the HUD!
	for i=1, #Characters do
		if ((Characters[i].X_Tile-1-OffsetX)*TileSize)+Characters[i].OffsetX >= 0
		and ((Characters[i].X_Tile-1-OffsetX)*TileSize)+Characters[i].OffsetX < ScreenSizeX*TileSize
		and ((Characters[i].Y_Tile-1-OffsetY)*TileSize)+Characters[i].OffsetY >= 0
		and ((Characters[i].Y_Tile-1-OffsetY)*TileSize)+Characters[i].OffsetY < ScreenSizeY*TileSize then
			--the actual drawing itself
			love.graphics.draw(TilesetPic, Tileset[Characters[i].Sprite], 
			((Characters[i].X_Tile-1-OffsetX)*TileSize)+Characters[i].OffsetX, 
			((Characters[i].Y_Tile-1-OffsetY)*TileSize)+Characters[i].OffsetY)
			
			love.graphics.print(Characters[i].Patrol_Storing, 
			((Characters[i].X_Tile-1-OffsetX)*TileSize)+Characters[i].OffsetX, 
			((Characters[i].Y_Tile-1-OffsetY)*TileSize)+Characters[i].OffsetY)			
		end
	end	
	
	--HUD elements
	love.graphics.draw(Picset[1], 0,0)
	
	--random text
	love.graphics.print(game_clock ,0,0)
end

function love.update(dt)
	--gameclock
	game_clock = game_clock + 1
	
	--key actions
	Repeated_Key_Behaviour() 
	
	--move all the chars
	for i=1, #Characters do
		if not Characters[i].Is_Moving then
			Patrol_Simple(Characters[i])	
		end		
	end		


	--doing smooth moving 
	if Player.Is_Moving then
		SmoothMoving_Player(Player.Mov_Dir, 8)
	end

	--smooth moving of characters
	for i=1, #Characters do
		if Characters[i].Is_Moving then
			SmoothMoving_General (Characters[i], Characters[i].Mov_Dir, Characters[i].Mov_Spd)		
		end
	end
end

--smooth moving
function SmoothMoving_Player (direction, speed)
	if math.abs(Player.OffsetX) == TileSize or math.abs(Player.OffsetY) == TileSize then
		Player.Is_Moving = false
		if direction == 'up' then 
			Player.OffsetY = 0 
			Player.Y_Tile = Player.Y_Tile - 1 
			if Player.Y_Tile == OffsetY + OffsetBorder 				--edge of screen
			and OffsetY > 0 								--within boundaries
			then OffsetY = OffsetY -Scrolling_Value end		--scrolling screen		
		elseif direction == 'down' then 
			Player.OffsetY = 0 
			Player.Y_Tile = Player.Y_Tile + 1
			if Player.Y_Tile == OffsetY + ScreenSizeY - OffsetBorder
			and OffsetY < SizeY - ScreenSizeY 
			then OffsetY = OffsetY +Scrolling_Value end				
		elseif direction == 'left' then 
			Player.OffsetX = 0 
			Player.X_Tile = Player.X_Tile - 1	
			if Player.X_Tile == OffsetX + OffsetBorder
			and OffsetX > 0 
			then OffsetX = OffsetX -Scrolling_Value end					
		elseif direction == 'right' then 
			Player.OffsetX = 0 
			Player.X_Tile = Player.X_Tile + 1 
			if Player.X_Tile == OffsetX + ScreenSizeX - OffsetBorder 
			and OffsetX < SizeX - ScreenSizeX 
			then OffsetX = OffsetX +Scrolling_Value end					
		end		
	else
		if direction == 'up' then 
			Player.OffsetY = Player.OffsetY - speed 
		elseif direction == 'down' then 
			Player.OffsetY = Player.OffsetY + speed  
		elseif direction == 'left' then 
			Player.OffsetX = Player.OffsetX - speed 
		elseif direction == 'right' then 
			Player.OffsetX = Player.OffsetX + speed 
		end				
	end
	

end

function SmoothMoving_General (character, direction, speed)
	if math.abs(character.OffsetX) == TileSize or math.abs(character.OffsetY) == TileSize then
		character.Is_Moving = false
		if direction == 'up' then 
			character.OffsetY = 0 
			character.Y_Tile = character.Y_Tile - 1 	
		elseif direction == 'down' then 
			character.OffsetY = 0 
			character.Y_Tile = character.Y_Tile + 1		
		elseif direction == 'left' then 
			character.OffsetX = 0 
			character.X_Tile = character.X_Tile - 1					
		elseif direction == 'right' then 
			character.OffsetX = 0 
			character.X_Tile = character.X_Tile + 1 			
		end		
	else
		if direction == 'up' then 
			character.OffsetY = character.OffsetY - speed 
		elseif direction == 'down' then 
			character.OffsetY = character.OffsetY + speed  
		elseif direction == 'left' then 
			character.OffsetX = character.OffsetX - speed 
		elseif direction == 'right' then 
			character.OffsetX = character.OffsetX + speed 
		end				
	end
	

end
































--possibly useful functions
function CountNeighbours(x,y, valueToCheck, Table)

	local asd=0

	if Table[x-1][y] == valueToCheck then asd=asd+1 end
	if Table[x+1][y] == valueToCheck then asd=asd+1 end
	if Table[x][y-1] == valueToCheck then asd=asd+1 end
	if Table[x][y+1] == valueToCheck then asd=asd+1 end	
	return asd
end

function FullNeighbours(x,y, valueToCheck, Table)
	local asd=0

	for j=-1,1 do
		for i=-1, 1 do
			if Table[x-j][y-i] == valueToCheck then asd=asd+1 end	
		end
	end
	
	return asd
end

function CountColors(Table, valueToCheck)

	local asd=0
	for j=2,127 do
		for i=2, 95 do
			if Table[j][i] == valueToCheck then asd=asd+1 end
		end
	end
	
	return asd
end

function ListColors(Table, valueToCheck)

	local list = {}
	for j=2,127 do
		for i=2, 95 do
			if Table[j][i] == valueToCheck then table.insert(list, {j,i}) end
		end
	end
	
	return list
end



function BackLoad(a,b)

	for j=1,128 do
		for i=1, 96 do
			a[j][i] = b[j][i]
		end
	end

end




